import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.StringTokenizer;
import java.text.SimpleDateFormat;


/**
 *
 * ComSkipMonitor is a Java program that monitors
 * file directories for mpg/mpeg files and runs
 * ComSkip on those files when they appear and are
 * no longer being written to.
 *
 * Revision history:
 *  Oct 3, 2006 steep: Initial version
 *  Oct 14, 2006 steep: Added Configuration servlet.
 */
public class ComskipMonitor
{
  private Properties props = new Properties();
  private List vidFiles = new ArrayList();
  private List cleanupFiles = new ArrayList();
  private List cleanupOnlyFiles = new ArrayList();
  private List batCleanupFiles = new ArrayList();
  private List relatedFiles = new ArrayList();
  private String threadnum = new String();
  private static final long oneMinute = 60000; //In milliseconds
  private static final int threadcount = 4; //In milliseconds
  private Thread[] processes;

  public static void main(String[] args)
  {
//      for (int i = 1; i <= threadcount; i++){
//          new ComskipMonitorThread(i + "").start();
//          try {
//              Thread.sleep((int)((Math.random() * 10000) + 10000));
//              } catch (InterruptedException e) {}
//      }
      
    System.out.println("Starting Monitor");

    ComskipMonitor csMon = new ComskipMonitor();
    synchronized (csMon)
    {
      while (true)
      {
        csMon.execute();
        try
        {
          csMon.wait(oneMinute * csMon.getTimeBetweenChecks());
        }
        catch (InterruptedException ie)
        {
          //If interupted, then continue
          break;
        }
      }
    }
  }

  /**
   * The constructor loads the properties
   */
  public ComskipMonitor()
  { 
    loadProperties();
    processes = new ProcessFilesThreaded[getMaxThreads() + 1];
  }
  /**
   * The constructor loads the properties
   */
  public ComskipMonitor(String str)
  {
    threadnum = str;  
    loadProperties();
    processes = new ProcessFilesThreaded[getMaxThreads() + 1];
  }

  /**
   * Initiates 1 pass-thru of comskip processing.
   * 1 call to execute will execute comskip on ALL
   * mpg/mpeg files that haven't been processed yet.
   * So the first time may take a while.
   *
   */
  public void execute()
  {
    if (shouldExecute())
    {
      initialize();
      getFiles();
      processFilesThreaded();
      cleanupFiles();
    }
    else
    {
      System.out.println("Execution excluded for this time of day.");
    }
  }

  /**
   * Resets the list of files.
   *
   */
  private void initialize()
  {
    vidFiles = new ArrayList();
    cleanupFiles = new ArrayList();
    cleanupOnlyFiles = new ArrayList();
    batCleanupFiles = new ArrayList();
    relatedFiles = new ArrayList();
  }
  
  /**
   * shouldExecute() checks to see whether ComSkipMonitor should process
   * its execute() method based upon the time of day.  The time of day options
   * can be configured in the properties file.
   * @return
   */
  private boolean shouldExecute()
  {
    boolean shouldExecute = true;
    if (props.containsKey("exclude.hours"))
    {
      Calendar cal = Calendar.getInstance();
      int hourOfDay = cal.get(Calendar.HOUR_OF_DAY);

      String excludeHours = props.getProperty("exclude.hours");
      StringTokenizer strTokenizer = new StringTokenizer(excludeHours, ",");
      while (strTokenizer.hasMoreTokens())
      {
        String hourString = strTokenizer.nextToken();
        if (String.valueOf(hourOfDay).equals(hourString.trim()))
        {
          shouldExecute = false;
        }
      }
    }
    return shouldExecute;
  }

  /**
   * Loads the properties file from the file csm.properties
   * which should exist in the same directory as ComSkipMonitor
   *
   */
  private void loadProperties()
  {
    try
    {
      File propFile = new File("../csm.properties");
      if (!propFile.exists())
      {
        alert("messages/missingProps.htm");
      }
      InputStream inStream = new FileInputStream(propFile);
      props.load(inStream);
    }
    catch (FileNotFoundException fnfe)
    {
      System.out.println("Unable to load csm.properties file, please ensure this file"
          + "exists in the same directory as ComSkipMonitor.");
    }
    catch (IOException ioe)
    {
      System.out.println("Unable to read csm.properties file.");
    }
  }

  /**
   * Generates three lists of files.  One list contains the names of the
   * mpg/mpeg files, and the other two contain all other files.  The Files
   * directories are configured in the properties file.
   *
   */
  private void getFiles()
  {
      String fileDirsProp = props.getProperty("file.directories");
      StringTokenizer tokens = new StringTokenizer(fileDirsProp, ",");
      while (tokens.hasMoreTokens())
      {
          getFiles(new File(tokens.nextToken().trim()));
      }
  }
  
  /**
   * Generates three lists of files.  One list contains the names of the
   * mpg/mpeg files and the other two contain all other files.  The Files
   * directories are configured in the properties file.
   *
   */
  private void getFiles(File fileDir)
  {
//    String fileDirsProp = props.getProperty("file.directories");
//    StringTokenizer tokens = new StringTokenizer(fileDirsProp, ",");
    String skipDirsProp = props.getProperty("skip.directories");
    if (skipDirsProp == null)
    {
        skipDirsProp = "";
        System.out.println("skip.directories property not found, defaulting to empty string");
    }
    List vidFileExtensions = getVideoFileExtensions();
    List cleanupFileExtensions = getCleanupFileExtensions();
    List cleanupOnlyFileExtensions = getExtraCleanupFileExtensions();
    List batCleanupFileExtensions = getBatCleanupFileExtensions();
    List relatedFileExtensions = getRelatedFileExtensions();
    boolean doRecurse = getRecurseSubdirs();

//    while (tokens.hasMoreTokens())
//    {
//      File fileDir = new File(tokens.nextToken().trim());
//      File fileDir = new File(fileFolder);
      if (fileDir.exists())
      {
        File[] files = fileDir.listFiles();
        for (int i = 0; i < files.length; i++)
        {
          
          if (files[i].isDirectory() && doRecurse)
          {
              StringTokenizer skipTokens = new StringTokenizer(skipDirsProp, ",");
              boolean dontSkip = true;
              while (skipTokens.hasMoreTokens())
              {
                  File skipFile = new File(skipTokens.nextToken().trim());
                  System.out.println("Testing for skip dir: " + files[i] + " -- "  + skipFile);
                  if(skipFile.equals(files[i]))
                  {
                      System.out.println("Directory " + files[i] + " skipped.");
                      dontSkip = false;
                      break;
                  }
              }
              if (dontSkip)
              {
                  getFiles(files[i]);
              }
          }
          
          boolean isVidFile = false;
          Iterator vidExtIter = vidFileExtensions.iterator();
          while (vidExtIter.hasNext())
          {
            String vidExt = (String) vidExtIter.next();
            if (files[i].getName().endsWith(vidExt))
            {
              vidFiles.add(files[i]);
              isVidFile = true;
            }
          }
          if (!isVidFile)
          {
            boolean isCleanupFile = false;
            Iterator cleanExtIter = cleanupFileExtensions.iterator();
            while (cleanExtIter.hasNext())
            {
              String cleanExt = (String) cleanExtIter.next();
              if (files[i].getName().endsWith(cleanExt))
              {
                cleanupFiles.add(files[i]);
                isCleanupFile = true;
              }
            }
            Iterator cleanOnlyExtIter = cleanupOnlyFileExtensions.iterator();
            while (cleanOnlyExtIter.hasNext())
            {
              String cleanOnlyExt = (String) cleanOnlyExtIter.next();
              if (files[i].getName().endsWith(cleanOnlyExt))
              {
                cleanupOnlyFiles.add(files[i]);
                isCleanupFile = true;
              }
            }
            Iterator batCleanExtIter = batCleanupFileExtensions.iterator();
            while (batCleanExtIter.hasNext())
            {
              String batCleanExt = (String) batCleanExtIter.next();
              if (files[i].getName().endsWith(batCleanExt))
              {
                  batCleanupFiles.add(files[i]);
                isCleanupFile = true;
              }
            }
            if (!isCleanupFile)
            {
              Iterator relatedExtIter = relatedFileExtensions.iterator();
              while (relatedExtIter.hasNext())
              {
                String relatedExt = (String) relatedExtIter.next();
                if (files[i].getName().endsWith(relatedExt))
                {
                  relatedFiles.add(files[i]);
                }
              }
            }
          }
        }
      }
      else
      {
        alert("messages/invalidDir.htm");
      }
//    }
  }

  /**
   * Loads the video extensions from the properties and puts them in a list.
   * @return List of video file extensions
   */
  private List getVideoFileExtensions()
  {
    List fileExtensions = new ArrayList();
    String prop = props.getProperty("process.file.extensions");
    StringTokenizer tokenizer = new StringTokenizer(prop, ",");
    while (tokenizer.hasMoreTokens())
    {
      fileExtensions.add(tokenizer.nextToken().trim());
    }
    return fileExtensions;
  }

  /**
   * Loads the cleanup extensions from the properties and puts them in a list.
   * @return List of cleanup file extensions
   */
  private List getCleanupFileExtensions()
  {
    List fileExtensions = new ArrayList();
    String prop = props.getProperty("cleanup.file.extensions");
    StringTokenizer tokenizer = new StringTokenizer(prop, ",");
    while (tokenizer.hasMoreTokens())
    {
      fileExtensions.add(tokenizer.nextToken().trim());
    }
    return fileExtensions;
  }
  
  /**
   * Loads the extra cleanup-only extensions from the properties and puts them in a list.
   * @return List of cleanup-only file extensions
   */
  private List getExtraCleanupFileExtensions()
  {
    List fileExtensions = new ArrayList();
    String prop = props.getProperty("extra.cleanup.only.file.extensions");
    if (prop == null)
    {
        prop = "";
        System.out.println("extra.cleanup.only.file.extensions property not found, defaulting to empty string");
    }
    StringTokenizer tokenizer = new StringTokenizer(prop, ",");
    while (tokenizer.hasMoreTokens())
    {
      fileExtensions.add(tokenizer.nextToken().trim());
    }
    return fileExtensions;
  }
  
  /**
   * Loads the extra cleanup-only extensions from the properties and puts them in a list.
   * @return List of cleanup-only file extensions
   */
  private List getBatCleanupFileExtensions()
  {
    List fileExtensions = new ArrayList();
    String prop = props.getProperty("bat.cleanup.extensions");
    if (prop == null)
    {
        prop = "";
        System.out.println("bat.extensions property not found, defaulting to empty string");
    }
    StringTokenizer tokenizer = new StringTokenizer(prop, ",");
    while (tokenizer.hasMoreTokens())
    {
      fileExtensions.add(tokenizer.nextToken().trim());
    }
    return fileExtensions;
  }

  /**
   * Loads the cleanup extensions from the properties and puts them in a list.
   * @return List of cleanup file extensions
   */
  private List getRelatedFileExtensions()
  {
    List fileExtensions = new ArrayList();
    String prop = props.getProperty("related.file.extensions");
    StringTokenizer tokenizer = new StringTokenizer(prop, ",");
    while (tokenizer.hasMoreTokens())
    {
      fileExtensions.add(tokenizer.nextToken().trim());
    }
    return fileExtensions;
  }

  /**
   * Display an error message to the user by sending an html
   * message file to the user.
   *
   */
  public void alert(String messageFile)
  {
    Runtime runtime = Runtime.getRuntime();
    File invalidDirectoryMessage = new File(messageFile);
    String[] cmd = new String[2];
    cmd[0] = "explorer";
    cmd[1] = invalidDirectoryMessage.getAbsolutePath();
    try
    {
      runtime.exec(cmd).waitFor();
    }
    catch (IOException ioe)
    {}
    catch (InterruptedException ie)
    {}
  }

  /**
   * processFiles() runs the comskip program on the mpg/mpeg
   * files that have not have comskip run on them.
   *
   */
  private void processFiles()
  {
    String comSkipLocation = props.getProperty("comskip.location");
    File comskipExeFile = new File(comSkipLocation);
    if (!comskipExeFile.exists())
    {
      alert("messages/missingExe.htm");
    }
    String comSkipConfig = props.getProperty("comskip.config");
    boolean isComskipExe = false;
    boolean isShowAnalyzerExe = false;
    if(comSkipLocation.endsWith("comskip.exe"))
    {
      isComskipExe = true;
      File comskipConfigFile = new File(comSkipConfig);
      if (!comskipConfigFile.exists())
      {
        alert("messages/missingIni.htm");
      }
    }
    else if(comSkipLocation.endsWith("ShowAnalyzer.exe"))
    {
      isShowAnalyzerExe = true;
    }
   

    Iterator fileIter = vidFiles.iterator();
    while (fileIter.hasNext() && shouldExecute())
    {
      //This loop processes the files in a batch
      //And it's possible that we can enter a time period
      //in which the user doesn't want comskip running
      //so we need to check the time after each file is
      //processed, not just at the beginning. thus shouldExecute() is checked.
      //Ditto for moving the slowCheck() inside of the loop.
      String[] cmd;
      int filenameArrayPosition = 2;
      List comskipParams;
      if(shouldRunSlow())
      {
        comskipParams = getComskipParameters(true);
        if(isComskipExe)
        {
          cmd = new String[4 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "--playnice";
          cmd[2] = "--ini=" + comSkipConfig;
          filenameArrayPosition = 3;
        }
        else if(isShowAnalyzerExe)
        {
          cmd = new String[6 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "-throttleLimit 33";
          cmd[2] = "-notrayIcon";
          cmd[3] = "-invisible";
          cmd[4] = "-close";
          filenameArrayPosition = 5;
        }
        else
        {
          cmd = new String[2 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          filenameArrayPosition = 1;
        }
      }
      else
      {
        if(isComskipExe)
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[3 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "--ini=" + comSkipConfig;
          filenameArrayPosition = 2;
        }
        else if(isShowAnalyzerExe)
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[5 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "-notrayIcon";
          cmd[2] = "-invisible";
          cmd[3] = "-close";
          filenameArrayPosition = 4;
        }
        else
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[2 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          filenameArrayPosition = 1;
        }
      }
      Iterator paramIter = comskipParams.iterator();
      while(paramIter.hasNext())
      {
        String param = (String)paramIter.next();
        cmd[filenameArrayPosition] = param;
        filenameArrayPosition++;
      }
      cmd[filenameArrayPosition] = "";

//      try {
//          Thread.sleep((int)((Math.random() * 20000) + 10000));
//          } catch (InterruptedException e) {}
      
      File file = (File) fileIter.next();
      if (shouldProcessFile(file))
      {
        cmd[filenameArrayPosition] = file.getAbsolutePath();
        Runtime runtime = Runtime.getRuntime();
        System.out.println("Execing " + cmd[0] + " " + " " + cmd[filenameArrayPosition] + " at " + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(Calendar.getInstance().getTime()) + " in thread " + threadnum);
        try
        {
          logProcessing(file.getAbsolutePath());
          Process process = runtime.exec(cmd);
          StreamGobbler errorGobbler = new StreamGobbler(process.getErrorStream(), "ERROR");
          StreamGobbler outputGobbler = new StreamGobbler(process.getInputStream(), "OUTPUT");

          errorGobbler.start();
          outputGobbler.start();

          int exitCode = process.waitFor();
          System.out.println("Exit Code:" + exitCode);
          cleanupLogs();
        }
        catch (IOException ioe)
        {
          System.out.println("Error executing comskip on file:" + file.getAbsolutePath());
        }
        catch (InterruptedException ie)
        {
          System.out.println("Interupted exception. Please report this bug.");
        }
        try {
            Thread.sleep((int)((Math.random() * 20000) + 10000));
            } catch (InterruptedException e) {}
      }
    }
  }
  
  /**
   * processFiles() runs the comskip program on the mpg/mpeg
   * files that have not have comskip run on them.
   *
   */
  private void processFilesThreaded()
  {
    String comSkipLocation = props.getProperty("comskip.location");
    String batLocation = props.getProperty("bat.location");
    File comskipExeFile = new File(comSkipLocation);
    if (!comskipExeFile.exists())
    {
      alert("messages/missingExe.htm");
    }
    String comSkipConfig = props.getProperty("comskip.config");
    boolean isComskipExe = false;
    boolean isShowAnalyzerExe = false;
    boolean batDefined = false;
    if(comSkipLocation.endsWith("comskip.exe"))
    {
      isComskipExe = true;
      File comskipConfigFile = new File(comSkipConfig);
      if (!comskipConfigFile.exists())
      {
        alert("messages/missingIni.htm");
      }
    }
    else if(comSkipLocation.endsWith("ShowAnalyzer.exe"))
    {
      isShowAnalyzerExe = true;
    }
    if(batLocation != null && batLocation.endsWith(".bat"))
    {
      File f = new File(batLocation);
      if ( f.exists() ){
          batDefined = true;
          System.out.println("Bat file " + batLocation + " defined and exists.");
      } else {
          System.out.println("Bat file " + batLocation + " defined but does not exist.");
      }
    }
   
    Iterator fileIter = vidFiles.iterator();
    
    while (fileIter.hasNext() && shouldExecute())
    {
      //This loop processes the files in a batch
      //And it's possible that we can enter a time period
      //in which the user doesn't want comskip running
      //so we need to check the time after each file is
      //processed, not just at the beginning. thus shouldExecute() is checked.
      //Ditto for moving the slowCheck() inside of the loop.
      String[] cmd;
      int filenameArrayPosition = 2;
      List comskipParams;
      if(shouldRunSlow())
      {
        comskipParams = getComskipParameters(true);
        if(isComskipExe)
        {
          cmd = new String[4 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "--playnice";
          cmd[2] = "--ini=" + comSkipConfig;
          filenameArrayPosition = 3;
        }
        else if(isShowAnalyzerExe)
        {
          cmd = new String[6 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "-throttleLimit 33";
          cmd[2] = "-notrayIcon";
          cmd[3] = "-invisible";
          cmd[4] = "-close";
          filenameArrayPosition = 5;
        }
        else
        {
          cmd = new String[2 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          filenameArrayPosition = 1;
        }
      }
      else
      {
        if(isComskipExe)
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[3 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "--ini=" + comSkipConfig;
          filenameArrayPosition = 2;
        }
        else if(isShowAnalyzerExe)
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[5 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          cmd[1] = "-notrayIcon";
          cmd[2] = "-invisible";
          cmd[3] = "-close";
          filenameArrayPosition = 4;
        }
        else
        {
          comskipParams = getComskipParameters(false);
          cmd = new String[2 + comskipParams.size()];
          cmd[0] = comSkipLocation;
          filenameArrayPosition = 1;
        }
      }
      Iterator paramIter = comskipParams.iterator();
      while(paramIter.hasNext())
      {
        String param = (String)paramIter.next();
        cmd[filenameArrayPosition] = param;
        filenameArrayPosition++;
      }
      cmd[filenameArrayPosition] = "";

//      try {
//          Thread.sleep((int)((Math.random() * 10000) + 10000));
//          } catch (InterruptedException e) {}
      
      File file = (File) fileIter.next();
      if (shouldProcessFile(file))
      {
          int myThread = findFreeThread();
          while(myThread == 0){
              System.out.println("All threads busy, waiting...");
            try {
            Thread.sleep((int)((Math.random() * 10000) + 5000));
            } catch (InterruptedException e) {}
            myThread = findFreeThread();
          }
          System.out.println("Proceeding with thread " + myThread);
          processes[myThread] = new ProcessFilesThreaded(myThread+"", this, file, cmd, filenameArrayPosition);
          processes[myThread].start();
      }
      if (batDefined && shouldBatProcessFile(file))
      {          
          cmd = new String[2];
          cmd[0] = batLocation;
          filenameArrayPosition = 1;
          
          int myThread = findFreeThread();
          while(myThread == 0){
              System.out.println("All threads busy, waiting...");
            try {
            Thread.sleep((int)((Math.random() * 10000) + 5000));
            } catch (InterruptedException e) {}
            myThread = findFreeThread();
          }
          System.out.println("Proceeding with thread " + myThread);
          processes[myThread] = new ProcessFilesThreaded(myThread+"", this, file, cmd, filenameArrayPosition);
          processes[myThread].start();
      }
    }
//    System.out.println("Waiting for all threads to complete.");
//    waitForAllThreads();
//    System.out.println("All threads complete!");
  }
  
  /**
   * Returns a list of comskip parameters from the properties
   * @return List of parameters
   */
  public List getComskipParameters(boolean removePlayNice)
  {
    List params = new ArrayList();
    String paramsProp = props.getProperty("comskip.parameters");
    StringTokenizer tokenizer = new StringTokenizer(paramsProp, ",");
    while(tokenizer.hasMoreTokens())
    {
      String param = tokenizer.nextToken().trim();
      //Duplicate command line params causes comskip.exe to error.
      if(removePlayNice)
      {
        if(!param.equals("-n") && !param.equals("--playnice"))
        {
          params.add(param);
        }
      }
      else
      {
        params.add(param);
      }
      
    }
    return params;
  }
  
  /**
   * Determines if --playnice attribute should be added to the Comskip 
   * command line.
   * @return boolean indicating whether or not to process the file slowly
   */
  private boolean shouldRunSlow()
  {
    boolean shouldRunSlow = false;
    if (props.containsKey("slow.hours"))
    {
      Calendar cal = Calendar.getInstance();
      int hourOfDay = cal.get(Calendar.HOUR_OF_DAY);

      String slowHours = props.getProperty("slow.hours");
      StringTokenizer strTokenizer = new StringTokenizer(slowHours, ",");
      while (strTokenizer.hasMoreTokens())
      {
        String hourString = strTokenizer.nextToken();
        if (String.valueOf(hourOfDay).equals(hourString.trim()))
        {
          shouldRunSlow = true;
        }
      }
    }
    return shouldRunSlow;
  }

  /**
   * Removes log files that are more than 3 days old.
   *
   */
  protected void cleanupLogs()
  {
    Calendar cal = Calendar.getInstance();
    List validLogs = new ArrayList();
    String filename = "ComSkip.";
    filename += (cal.get(Calendar.MONTH) + 1) + "-";
    filename += cal.get((Calendar.DAY_OF_MONTH)) + "-";
    filename += cal.get(Calendar.YEAR) + ".log";
    validLogs.add(filename);
    filename = new String("ComSkip.");
    cal.add(Calendar.DAY_OF_MONTH, -1);
    filename += (cal.get(Calendar.MONTH) + 1) + "-";
    filename += cal.get((Calendar.DAY_OF_MONTH)) + "-";
    filename += cal.get(Calendar.YEAR) + ".log";
    validLogs.add(filename);
    filename = new String("ComSkip.");
    cal.add(Calendar.DAY_OF_MONTH, -1);
    filename += (cal.get(Calendar.MONTH) + 1) + "-";
    filename += cal.get((Calendar.DAY_OF_MONTH)) + "-";
    filename += cal.get(Calendar.YEAR) + ".log";
    validLogs.add(filename);
    File currentDir = new File(".\\");
    File[] files = currentDir.listFiles();
    for (int i = 0; i < files.length; i++)
    {
      boolean validLog = false;
      File file = files[i];
      if (file.getAbsolutePath().endsWith(".log"))
      {
        Iterator validLogIter = validLogs.iterator();
        while (validLogIter.hasNext())
        {
          String validLogName = (String) validLogIter.next();
          if (file.getAbsolutePath().indexOf(validLogName) >= 0)
          {
            validLog = true;
          }
        }
        if (!validLog)
        {
          file.delete();
        }
      }
    }
  }

  /**
   * Logs the files that comskip processes.
   * The name of the log file is Comskip.date.log.
   * Log files older than 3 days are deleted.
   * @param filePath
   */
  protected void logProcessing(String filePath)
  {
    Calendar cal = Calendar.getInstance();
    String logFileName = "ComSkip.";
    String currentDate = "";
    currentDate += (cal.get(Calendar.MONTH) + 1) + "-";
    currentDate += cal.get(Calendar.DAY_OF_MONTH) + "-";
    currentDate += cal.get(Calendar.YEAR);
    logFileName += currentDate + ".log";
    File logFile = new File(logFileName);
    if (!logFile.exists())
    {
      try
      {
        logFile.createNewFile();
      }
      catch (IOException ioe)
      {
        System.out.println("Unable to create log file");
      }
    }
    try
    {
      FileOutputStream outStream = new FileOutputStream(logFile, true);
      String tempString = String.valueOf(cal.get(Calendar.HOUR_OF_DAY));
      if(tempString.length() < 2)
      {
        tempString = "0" + tempString;
      }
      tempString = String.valueOf(cal.get(Calendar.HOUR_OF_DAY));
      currentDate += " " + tempString;
      tempString = String.valueOf(cal.get(Calendar.MINUTE));
      if(tempString.length() < 2)
      {
        tempString = "0" + tempString;
      }
      currentDate += ":" + tempString + " ";
      outStream.write((currentDate + "Processing file: " + filePath + "\r\n").getBytes());
      outStream.close();
    }
    catch (IOException ioe)
    {
      System.out.println("Unable to create log file");
    }
  }

  /**
   * cleanupFiles() removes the comskip generated log and text files
   * that correspond to mpg/mpeg files that no longer exist.
   *
   */
  private void cleanupFiles()
  {
    Iterator fileIter = cleanupFiles.iterator();
    while (fileIter.hasNext())
    {
      File file = (File) fileIter.next();
      if (shouldDeleteFile(file))
      {
        file.delete();
      }
    }
    fileIter = cleanupOnlyFiles.iterator();
    while (fileIter.hasNext())
    {
      File file = (File) fileIter.next();
      if (shouldDeleteFile(file))
      {
        file.delete();
      }
    }
    fileIter = batCleanupFiles.iterator();
    while (fileIter.hasNext())
    {
      File file = (File) fileIter.next();
      if (shouldDeleteFile(file))
      {
        file.delete();
      }
    }
  }

  /**
   * Evaluates whether or not a comskip generated file should be deleted.
   * If a comskip file exists with no video sibling, it is assumed
   * to have been deleted by the PVR software.
   * @param file
   * @return true if the file should be deleted
   */
  private boolean shouldDeleteFile(File file)
  {
    boolean shouldDelete = true;
    
    List filesToCheck = new ArrayList();
    filesToCheck.addAll(vidFiles);
    filesToCheck.addAll(relatedFiles);
    Iterator fileIter = filesToCheck.iterator();
    while (fileIter.hasNext() && shouldDelete)
    {
      File fileToCheck = (File) fileIter.next();
      String filenamePrefix1 = getFilenamePrefix(fileToCheck);
      String filenamePrefix2 = getFilenamePrefix(file);
      System.out.println(filenamePrefix1 + " -- "  + filenamePrefix2);
      if(filenamePrefix1.equals(filenamePrefix2))
      {
        shouldDelete = false;
      }
    }

    return shouldDelete;
  }

  /**
   * Analyzes a string and returns its file extension
   * @param file
   */
  private String getFilenamePrefix(File file)
  {
    String filename = file.getAbsolutePath();
    int dotIndex = filename.lastIndexOf(".");
    return filename.substring(0, dotIndex);
  }
  
  /**
   * Evaluates whether or not an mpg/mpeg file has had comskip
   * run on it.
   * @param file
   * @return true if the file should be processed
   */
  private boolean shouldProcessFile(File file)
  {
//      List localCleanupFiles = new ArrayList();
//      {
//    String fileDirsProp = props.getProperty("file.directories");
//    StringTokenizer tokens = new StringTokenizer(fileDirsProp, ",");
//    List cleanupFileExtensions = getCleanupFileExtensions();
//
//    while (tokens.hasMoreTokens())
//    {
//      File fileDir = new File(tokens.nextToken().trim());
//      if (fileDir.exists())
//      {
//        File[] files = fileDir.listFiles();
//        for (int i = 0; i < files.length; i++)
//        {
//            boolean isCleanupFile = false;
//            Iterator cleanExtIter = cleanupFileExtensions.iterator();
//            while (cleanExtIter.hasNext())
//            {
//              String cleanExt = (String) cleanExtIter.next();
//              if (files[i].getName().endsWith(cleanExt))
//              {
//                localCleanupFiles.add(files[i]);
//                isCleanupFile = true;
//              }
//            }
//        }
//      }
//      else
//      {
//        alert("messages/invalidDir.htm");
//      }
//    }
//      }
      
    boolean shouldProcess = true;
    
    List filesToCheck = new ArrayList();
    filesToCheck.addAll(cleanupFiles);
    Iterator fileIter = filesToCheck.iterator();
    while (fileIter.hasNext() && shouldProcess)
    {
      File fileToCheck = (File) fileIter.next();
      String filenamePrefix1 = getFilenamePrefix(fileToCheck);
      String filenamePrefix2 = getFilenamePrefix(file);
      System.out.println(filenamePrefix1 + " -- "  + filenamePrefix2);
      if(filenamePrefix1.equals(filenamePrefix2))
      {
        shouldProcess = false;
      }
    }

    if (shouldProcess)
    {
      String continuous = props.getProperty("execute.during.record");
      if(continuous != null && continuous.equals("no"))
      //Else process the file even if it's still recording.
      {
        //LastModified doesn't get continualy updated
        //as the file is being written to.
        //However, a file that is being written to
        //should be growing in size.
        //Thus.. the following check.
        synchronized (file)
        {
          long fileLength = file.length();
          try
          {
            file.wait(oneMinute / 6); //10 seconds
          }
          catch (InterruptedException ie)
          {}
          shouldProcess = (fileLength == file.length());
        }
      }
    }

    return shouldProcess;
  }
  
  /**
   * Evaluates whether or not an mpg/mpeg file has had bat file
   * run on it.
   * @param file
   * @return true if the file should be processed
   */
  private boolean shouldBatProcessFile(File file)
  {
      
    boolean shouldProcess = true;
    
    
    List filesToCheck = new ArrayList();
    filesToCheck.addAll(batCleanupFiles);

    Iterator fileIter = filesToCheck.iterator();
    while (fileIter.hasNext() && shouldProcess)
    {
      File fileToCheck = (File) fileIter.next();
      String filenamePrefix1 = getFilenamePrefix(fileToCheck);
      String filenamePrefix2 = getFilenamePrefix(file);
      System.out.println(filenamePrefix1 + " -- "  + filenamePrefix2);
      if(filenamePrefix1.equals(filenamePrefix2))
      {
        shouldProcess = false;
      }
    }

    if (shouldProcess)
    {
      String continuous = props.getProperty("execute.during.record.bat");
      if(continuous != null && continuous.equals("no"))
      //Else process the file even if it's still recording.
      {
        //LastModified doesn't get continualy updated
        //as the file is being written to.
        //However, a file that is being written to
        //should be growing in size.
        //Thus.. the following check.
        synchronized (file)
        {
          long fileLength = file.length();
          try
          {
            file.wait(oneMinute / 6); //10 seconds
          }
          catch (InterruptedException ie)
          {}
          shouldProcess = (fileLength == file.length());
        }
      }
    }

//    if (shouldProcess){
//        System.out.println("shouldBatProcessFile returning true for " + file.getName());
//    }else{
//        System.out.println("shouldBatProcessFile returning false for " + file.getName());
//    }
    return shouldProcess;
  }

  public int getTimeBetweenChecks()
  {
    String timeBetweenChecks = props.getProperty("time.between.checks");
    return Integer.parseInt(timeBetweenChecks);
  }
  
  public int getMaxThreads()
  {
    String maxThreads = props.getProperty("max.threads");
    if (maxThreads == null){
        maxThreads = "2";
        System.out.println("max.threads not set, defaulting to " + maxThreads);
    }
    return Integer.parseInt(maxThreads);
  }
  
  public boolean getRecurseSubdirs()
  {
    String doRecurse = props.getProperty("recurse.subdirectories");
    if (doRecurse == null){
        doRecurse = "false";
        System.out.println("recurse.subdirectories not set, defaulting to " + doRecurse);
    }
    return Boolean.parseBoolean(doRecurse);
  }
  
  private int findFreeThread(){
      int freeThread = 0;
      for (int i = 1; i <= getMaxThreads(); i++){
          if (processes[i] == null){
              freeThread = i;
              break;
          }
          try {
              processes[i].join(100);
          } catch (InterruptedException e) {}

          if (processes[i].isAlive()){
              continue;
          }
          freeThread = i;
      }
      return freeThread;
  }
  
  private void waitForAllThreads(){
      for (int i = 1; i <= getMaxThreads(); i++){
          if (processes[i] == null){
              continue;
          }
          try {
              processes[i].join();
          } catch (InterruptedException e) {}
      }
  }
}

